@@ -1,6 +1,6 @@
 package POE::API::Peek;
-BEGIN {
-  $POE::API::Peek::VERSION = '2.16';
+{
+  $POE::API::Peek::VERSION = '2.18';
 }
 # ABSTRACT: Peek into the internals of a running POE environment
 
@@ -14,8 +14,8 @@ BEGIN {
 	use POE;
 	my $ver = $POE::VERSION;
 	$ver =~ s/_.+$//;
-	if($ver < '1.293') {
-		die(__PACKAGE__." is only certified for POE version 1.293 and up and you are running POE version " . $ver . ". Check CPAN for an appropriate version of ".__PACKAGE__.".");
+	if($ver < '1.300') {
+		die(__PACKAGE__." is only certified for POE version 1.300 and up and you are running POE version " . $ver . ". Check CPAN for an appropriate version of ".__PACKAGE__.".");
 	}
 }
 
@@ -29,12 +29,12 @@ our @CARP_NOT = qw(__PACKAGE__);
 # new {{{
 
 
-sub new { 
+sub new {
 	my $class = shift;
 	my $self = {
 		broken_event_queue_bitch => 0,
 	};
-	return bless $self, $class; 
+	return bless $self, $class;
 }
 
 # }}}
@@ -67,8 +67,8 @@ sub is_kernel_running {
 # active_event {{{
 
 
-sub active_event { 
-	return ${ $poe_kernel->[ POE::Kernel::KR_ACTIVE_EVENT() ] }; 
+sub active_event {
+	return ${ $poe_kernel->[ POE::Kernel::KR_ACTIVE_EVENT() ] };
 }
 
 #}}}
@@ -86,7 +86,7 @@ sub kernel_memory_size {
 
 sub event_list {
 	my $self = shift;
-   
+
 	my %events;
 	foreach my $session_ref (keys %{ $poe_kernel->[ &POE::Kernel::KR_SESSIONS() ] }) {
 		my $session = $poe_kernel->[ &POE::Kernel::KR_SESSIONS() ]->{ $session_ref }->[ &POE::Kernel::SS_SESSION() ];
@@ -120,7 +120,7 @@ sub which_loop {
 # current_session {{{
 
 
-# the value of KR_ACTIVE_SESSION is a ref to a scalar. so we deref it before 
+# the value of KR_ACTIVE_SESSION is a ref to a scalar. so we deref it before
 # handing it to the user.
 
 sub current_session { return ${ $poe_kernel->[POE::Kernel::KR_ACTIVE_SESSION] } }
@@ -133,7 +133,8 @@ sub current_session { return ${ $poe_kernel->[POE::Kernel::KR_ACTIVE_SESSION] }
 sub get_session_children {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ses_get_children($session);
+	my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ses_get_children($sid);
 }
 # }}}
 
@@ -143,8 +144,10 @@ sub get_session_children {
 sub is_session_child {
 	my $self = shift;
 	my $parent = shift or return undef;
+	my $psid = ref $parent ? $parent->ID : $parent;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ses_is_child($parent, $session);
+	my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ses_is_child($psid, $sid);
 }
 # }}}
 
@@ -154,7 +157,8 @@ sub is_session_child {
 sub get_session_parent {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ses_get_parent($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ses_get_parent($sid);
 }
 # }}}
 
@@ -164,8 +168,8 @@ sub get_session_parent {
 
 sub resolve_session_to_ref {
 	my $self = shift;
-	my $id = shift || $self->current_session()->ID;
-	return $poe_kernel->_data_sid_resolve($id);
+	my $sid = shift || $self->current_session()->ID;
+	return $poe_kernel->_data_sid_resolve($sid);
 }
 # }}}
 
@@ -185,7 +189,8 @@ sub resolve_session_to_id {
 sub get_session_refcount {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ses_refcount($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ses_refcount($sid);
 }
 # }}}
 
@@ -217,6 +222,7 @@ sub session_list {
 sub session_memory_size {
 	my $self = shift;
 	my $session = shift || $self->current_session();
+    $session = $poe_kernel->_data_sid_resolve($session) unless ref $session;
 	return total_size($session);
 }
 # }}}}
@@ -258,7 +264,8 @@ sub resolve_alias {
 sub session_alias_list {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_alias_list($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_alias_list($sid);
 }
 # }}}
 
@@ -268,7 +275,8 @@ sub session_alias_list {
 sub session_alias_count {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_alias_count_ses($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_alias_count_ses($sid);
 }
 # }}}
 
@@ -278,7 +286,8 @@ sub session_alias_count {
 sub session_id_loggable {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_alias_loggable($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_alias_loggable($sid);
 }
 # }}}
 
@@ -290,7 +299,8 @@ sub session_id_loggable {
 sub event_count_to {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ev_get_count_to($session);    
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ev_get_count_to($sid);
 }
 #}}}
 
@@ -300,7 +310,8 @@ sub event_count_to {
 sub event_count_from {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_ev_get_count_from($session);    
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_ev_get_count_from($sid);
 }
 
 #}}}
@@ -308,8 +319,8 @@ sub event_count_from {
 # event_queue {{{
 
 
-sub event_queue { 
-	return $poe_kernel->[POE::Kernel::KR_QUEUE] 
+sub event_queue {
+	return $poe_kernel->[POE::Kernel::KR_QUEUE]
 }
 
 # }}}
@@ -317,7 +328,7 @@ sub event_queue {
 # event_queue_dump {{{
 
 
-sub event_queue_dump { 
+sub event_queue_dump {
 	my $self = shift;
 	my $queue = $self->event_queue;
 
@@ -328,10 +339,10 @@ sub event_queue_dump {
 	foreach my $qitem (@queue) {
 		my $item = {};
 		my ($priority, $id, $payload) = @$qitem;
-		 
+
 		$item->{ID} = $id;
 		$item->{index} = $i++;
-		$item->{priority} = $priority; 
+		$item->{priority} = $priority;
 
 		my $ev_name = $payload->[POE::Kernel::EV_NAME()];
 		$item->{event} = $ev_name;
@@ -367,7 +378,7 @@ sub event_queue_dump {
 				$type_str = 'Unknown';
 			}
 		}
-        
+
 		$item->{type} = $type_str;
 		push @happy_queue, $item;
 	}
@@ -396,7 +407,8 @@ sub extref_count {
 sub get_session_extref_count {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_extref_count_ses($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_extref_count_ses($sid);
 }
 # }}}
 
@@ -428,12 +440,30 @@ sub handle_count {
 sub session_handle_count {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_handle_count_ses($session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_handle_count_ses($sid);
 }
 # }}}
 
 # }}}
 
+# PID Fun {{{
+
+# session_pid_count {{{
+
+
+sub session_pid_count {
+    my $self = shift;
+    my $session = shift || $self->current_session();
+    my $sid = ref $session ? $session->ID : $session;
+    return $poe_kernel->_data_sig_pids_ses($sid);
+}
+
+# }}}
+
+# }}}
+
+
 # Signals Fun {{{
 
 
@@ -471,7 +501,8 @@ sub is_signal_watched {
 sub signals_watched_by_session {
 	my $self = shift;
 	my $session = shift || $self->current_session();
-	my %sigs = $poe_kernel->_data_sig_watched_by_session($session);
+    my $sid = ref $session ? $session->ID : $session;
+	my %sigs = $poe_kernel->_data_sig_watched_by_session($sid);
 
 	my %ret;
 	foreach my $k (keys %sigs) {
@@ -494,7 +525,7 @@ sub signal_watchers {
 	my %ret;
 	foreach my $k (keys %sigs) {
 		my $ev = $sigs{$k}[0];
-		$ret{$k} = $ev;
+		$ret{$poe_kernel->alias_resolve($k)} = $ev;
 	}
 
 	return %ret;
@@ -508,7 +539,8 @@ sub is_signal_watched_by_session {
 	my $self = shift;
 	my $signal = shift or return undef;
 	my $session = shift || $self->current_session();
-	return $poe_kernel->_data_sig_is_watched_by_session($signal, $session);
+    my $sid = ref $session ? $session->ID : $session;
+	return $poe_kernel->_data_sig_is_watched_by_session($signal, $sid);
 }
 # }}}
 
@@ -526,7 +558,7 @@ POE::API::Peek - Peek into the internals of a running POE environment
 
 =head1 VERSION
 
-version 2.16
+version 2.18
 
 =head1 DESCRIPTION
 
@@ -535,11 +567,11 @@ to Kernel internals in a cross-version compatible manner. Other
 calculated data is also available.
 
 My intention is to provide massive amounts of internal data for use in
-POE debugging. 
+POE debugging.
 
 =head1 WARNING
 
-B<This version of this module is certified against POE version 1.293 and 
+B<This version of this module is certified against POE version 1.300 and
 above. It will fail on any other POE version.>
 
 B<Further, this module requires perl v5.6.1 or above.>
@@ -586,7 +618,7 @@ scalar containing a string.
       # do stuff...
   }
 
-Tell if the POE Kernel is running and active. Returns 1 if the Kernel is 
+Tell if the POE Kernel is running and active. Returns 1 if the Kernel is
 running and 0 if it is not.
 
 =cut
@@ -622,7 +654,7 @@ See the Devel::Size documentation for several caveats involved in this metric.
   my $events = $api->event_list();
 
 Gets the list of events for the whole POE environment. Returns a hash
-with the session IDs as the keys and a list of events as the 
+with the session IDs as the keys and a list of events as the
 values.
 
 =cut
@@ -662,11 +694,11 @@ parameters. Returns a scalar containing a reference.
 
 =head2 get_session_children
 
-    my @children = $api->get_session_children($session);
+    my @children = $api->get_session_children($session_id);
     my @children = $api->get_session_children();
 
 Get the children (if any) for a given session. Takes one optional
-parameter, a POE::Session object. If this parameter is not provided, the
+parameter, a POE::Session object or ID. If this parameter is not provided, the
 method defaults to the currently active session. Returns a list of
 POE::Session objects.
 
@@ -677,6 +709,7 @@ POE::Session objects.
 
 =head2 is_session_child
 
+  if($api->is_session_child($parent, $session_id)) { }
   if($api->is_session_child($parent, $session)) { }
   if($api->is_session_child($parent)) { }
 
@@ -685,7 +718,7 @@ mandatory parameter, a POE::Session object which is the potential parent
 session this method will interrogate. Takes one optional parameter, a
 POE::Session object which is the session whose parentage this method
 will determine. If this parameter is not specified, it will default to
-the currently active session. Returns a boolean. 
+the currently active session. Returns a boolean.
 
 =cut
 
@@ -694,12 +727,13 @@ the currently active session. Returns a boolean.
 
 =head2 get_session_parent
 
+    my $parent = $api->get_session_parent($session_id);
     my $parent = $api->get_session_parent($session);
     my $parent = $api->get_session_parent();
 
 Get the parent for a given session. Takes one optional parameter, a
-POE::Session object. If this parameter is not provided, the method defaults to
-the currently active session. Returns a POE::Session object.
+POE::Session object or ID. If this parameter is not provided, the method
+defaults to the currently active session. Returns a POE::Session object.
 
 =cut
 
@@ -727,7 +761,7 @@ POE::Session object on success; undef on failure.
   my $session_id = $api->resolve_session_to_id();
 
 Obtain the session id for a given POE::Session object. Takes one
-optional parameter, a POE::Session object. If this parameter is not
+optional parameter, a POE::Session object or ID. If this parameter is not
 specified, it will default to the currently active session. Returns an
 integer on success and undef on failure.
 
@@ -738,11 +772,12 @@ integer on success and undef on failure.
 
 =head2 get_session_refcount
 
+  my $count = $api->get_session_refcount($session_id);
   my $count = $api->get_session_refcount($session);
   my $count = $api->get_session_refcount();
 
 Obtain the reference count for a given POE::Session. Takes one optional
-parameter, a POE::Session object. If this parameter is not specified, it
+parameter, a POE::Session object or ID. If this parameter is not specified, it
 will default to the currently active session. Returns an integer.
 
 =cut
@@ -757,7 +792,7 @@ will default to the currently active session. Returns an integer.
 Obtain a count of how many sessions exist. Takes no parameters. Returns
 an integer.
 
-Note: for various reasons, the Kernel counts as a session. 
+Note: for various reasons, the Kernel counts as a session.
 
 =cut
 
@@ -783,9 +818,10 @@ from this list.
 
   my $size = $api->session_memory_size();
   my $size = $api->session_memory_size($session);
+  my $size = $api->session_memory_size($session_id);
 
 Get the memory footprint of a session. If no session is provided, the current
-session is used. See the Devel::Size documentation for several caveats 
+session is used. See the Devel::Size documentation for several caveats
 involved in this metric.
 
 =cut
@@ -799,6 +835,8 @@ involved in this metric.
   my $events = $api->session_event_list();
   my @events = $api->session_event_list($session);
   my $events = $api->session_event_list($session);
+  my @events = $api->session_event_list($session_id);
+  my $events = $api->session_event_list($session_id);
 
 Get the list of events for a session. If no session is provided, the current
 session is used.
@@ -829,11 +867,12 @@ undef on failure.
 
 =head2 session_alias_list
 
+  my @aliases = $api->session_alias_list($session_id);
   my @aliases = $api->session_alias_list($session);
   my @aliases = $api->session_alias_list();
 
 Obtain a list of aliases for a POE::Session object. Takes one optional
-parameter, a POE::Session object. If this parameter is not specified, it
+parameter, a POE::Session object or ID. If this parameter is not specified, it
 will default to the currently active session. Returns a list of strings.
 
 =cut
@@ -843,11 +882,12 @@ will default to the currently active session. Returns a list of strings.
 
 =head2 session_alias_count
 
+  my $count = $api->session_alias_count($session_id);
   my $count = $api->session_alias_count($session);
   my $count = $api->session_alias_count();
 
 Obtain the count of how many aliases a session has. Takes one optional
-parameter, a POE::Session object. If this parameter is not specified, it
+parameter, a POE::Session object or ID. If this parameter is not specified, it
 will default to the currently active session. Returns an integer.
 
 =cut
@@ -857,11 +897,12 @@ will default to the currently active session. Returns an integer.
 
 =head2 session_id_loggable
 
+    my $str = $api->session_id_loggable($session_id);
     my $str = $api->session_id_loggable($session);
     my $str = $api->session_id_loggable();
 
 Obtain a loggable version of a session id. Takes one optional parameter,
-a POE::Session object. If this parameter is not specified, it will
+a POE::Session object or ID. If this parameter is not specified, it will
 default to the currently active session. Returns a string.
 
 =cut
@@ -874,11 +915,12 @@ default to the currently active session. Returns a string.
 
 =head2 event_count_to
 
+  my $count = $api->event_count_to($session_id);
   my $count = $api->event_count_to($session);
   my $count = $api->event_count_to();
 
 Get the number of events heading toward a particular session. Takes one
-parameter, a POE::Session object. if none is provided, defaults to the
+parameter, a POE::Session object or ID. if none is provided, defaults to the
 current session. Returns an integer.
 
 =cut
@@ -888,11 +930,12 @@ current session. Returns an integer.
 
 =head2 event_count_from
 
+  my $count = $api->get_session_extref_count($session_id);
   my $count = $api->event_count_from($session);
   my $count = $api->event_count_from();
 
 Get the number of events heading out from a particular session. Takes one
-parameter, a POE::Session object. If non is provided, defaults to the 
+parameter, a POE::Session object or ID. If none is provided, defaults to the
 current session. Return an integer.
 
 =cut
@@ -922,7 +965,7 @@ following entries:
 
 =over 4
 
-=item * ID 
+=item * ID
 
 The id number that POE's queue identifies this entry as.
 
@@ -950,7 +993,7 @@ Where this event is headed. Usually a POE::Session.
 
 The type of event this is. May have the value User, _start, _stop, _signal,
 _garbage_collect, _parent, _child, _sigchld_poll, Alarm, File Activity, or
-Unknown. 
+Unknown.
 
 =back
 
@@ -979,11 +1022,12 @@ Returns an integer.
 
 =head2 get_session_extref_count
 
+  my $count = $api->get_session_extref_count($session_id);
   my $count = $api->get_session_extref_count($session);
   my $count = $api->get_session_extref_count();
 
 Obtain the number of extra references a session has. Takes one optional
-parameter, a POE::Session object. If this parameter is not specified, it
+parameter, a POE::Session object or ID. If this parameter is not specified, it
 will default to the currently active session. Returns an integer.
 
 =cut
@@ -1021,13 +1065,29 @@ Returns an integer.
 =pod
 
 
-=head2 session_handle_count 
+=head2 session_handle_count
 
+  my $count = $api->session_handle_count($session_id);
   my $count = $api->session_handle_count($session);
   my $count = $api->session_handle_count();
 
 Obtain a count of the active handles for a given session. Takes one
-optional parameter, a POE::Session object. If this parameter is not
+optional parameter, a POE::Session object or ID. If this parameter is not
+supplied, it will default to the currently active session.
+
+=cut
+
+=pod
+
+
+=head2 session_pid_count
+
+    my $count = $api->session_pid_count($session_id);
+    my $count = $api->session_pid_count($session);
+    my $count = $api->session_pid_count();
+
+Obtain a count of the process IDs being watched by a session. Takes one
+optional parameter, a POE::Session object or ID. If this parameter is not
 supplied, it will default to the currently active session.
 
 =cut
@@ -1094,7 +1154,7 @@ parameter, a signal name. Returns a boolean.
   my %signals = $api->signals_watched_by_session();
 
 Get the signals watched by a session and the events they generate. Takes
-one optional parameter, a POE::Session object. If this parameter is not
+one optional parameter, a POE::Session object or ID. If this parameter is not
 supplied, it will default to the currently active session. Returns a
 hash, with a signal name as the key and the event the session generates
 as the value.
@@ -1119,22 +1179,25 @@ reference with an event name as the value.
 
 =head2 is_signal_watched_by_session
 
+  if($api->is_signal_watched_by_session($signal_name, $session_id)) { }
   if($api->is_signal_watched_by_session($signal_name, $session)) { }
   if($api->is_signal_watched_by_session($signal_name)) { }
 
 Determine if a given session is explicitly watching a signal. Takes one
 mandatory parameter, a signal name. Takes one optional parameter, a
-POE::Session object. If this parameter is not provided, it will default
+POE::Session object or ID. If this parameter is not provided, it will default
 to the currently active session. Returns a boolean.
 
 =head1 AUTHORS
 
 sungo <sungo@sungo.us>
 Yuval Kogman <nothingmuch@woobling.org>
+Chris 'BinGOs' Williams <bingos@cpan.org>
+Philip Gwyn <gwyn@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2010 by Matt Cashner (sungo).
+This software is Copyright (c) 2011 by Matt Cashner (sungo).
 
 This is free software, licensed under:
 
